/*###############################################################################
# Exemplo para criação de tasks - KIT: WEMOS D1 MINI ESP32
# Esse exemplo mostra como criar tarefas no FreeRTOS
# Exemplo 01: Tasks / Projeto / Platformio / FreeRTOS
# Por: Prof. Dr. Paulo Denis Garcez da Luz
# http://www.elf74.daeln.com.br
###############################################################################*/

/* Biblioteca do Arduino */
#include <Arduino.h>

/* Bibliotecas FreeRTOS */
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"

 /* Mapeamento de Pinos */ 
#define LEDBUILTIN 2
#define PUSHBUTTON 12
#define LEDPCBEXT1 14
#define ADCPOTPIN1 34

/* Variáveis para armazenamento do handle das tasks */
TaskHandle_t taks1Handle = NULL;
TaskHandle_t taks2Handle = NULL;

/* Protótipos das Tasks */
void vTask1(void *pvParameters);
void vTask2(void *pvParameters);

/* Função Setup */
void setup()
{
  /* configura pino 2 e 14 como saída*/
  pinMode(LEDBUILTIN,OUTPUT);
  pinMode(LEDPCBEXT1,OUTPUT);
  /* desliga os Leds da PCB */
  digitalWrite(LEDBUILTIN,LOW);
  digitalWrite(LEDPCBEXT1,LOW);
  /* configura comunicação serial com baudrate de 115200 */
  Serial.begin(115200);
  /* Criação das Tasks */
  xTaskCreate(vTask1,"vTASK1",configMINIMAL_STACK_SIZE,NULL,1,&taks1Handle);
  xTaskCreate(vTask2,"vTASK2",configMINIMAL_STACK_SIZE+512,NULL,2,&taks2Handle);
}

/* Função Loop */
void loop()
{
  vTaskDelay(3000); /* libera a CPU por 3 seg */
}

/* vTask1 => inverte LED em intervalos de 200 ms */
void vTask1(void *pvParameters)
{
  while (1)
        {
          digitalWrite(LEDBUILTIN,!digitalRead(LEDBUILTIN));
          vTaskDelay(pdMS_TO_TICKS(200));
        }
}

/* vTask2 => imprime valor de contagem a cada 1 seg */
void vTask2(void *pvParameters)
{
  int cont = 0;
  while (1)
        {
          Serial.println("vTask 2: " + String(cont++));
          vTaskDelay(pdMS_TO_TICKS(1000));
        }
}