/*###############################################################################
# Exemplo para manipulação de tasks - KIT: WEMOS D1 MINI ESP32
# Esse exemplo mostra como passar parâmetros para as Tasks no FreeRTOS
# Exemplo 04: Tasks / Platformio / FreeRTOS
# Por: Prof. Dr. Paulo Denis Garcez da Luz
# http://www.elf74.daeln.com.br
###############################################################################*/

/* Biblioteca do Arduino */
#include <Arduino.h>

/* Bibliotecas FreeRTOS */
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"

 /* Mapeamento de Pinos */ 
#define LEDBUILTIN 2
#define PUSHBUTTON 12
#define LEDPCBEXT1 14
#define ADCPOTPIN1 34

/* Variáveis para armazenamento do handle das tasks */
TaskHandle_t task1Handle = NULL;
TaskHandle_t task2Handle = NULL;

/* Variáveis Globais*/
int valorinicial = 500;

/* Protótipos das Tasks */
void vTask1(void *pvParameters);
void vTask2(void *pvParameters);

/* Função Setup */
void setup()
{
  /* configura pino 2 e 14 como saída*/
  pinMode(LEDBUILTIN,OUTPUT);
  pinMode(LEDPCBEXT1,OUTPUT);
  /* desliga os Leds da PCB */
  digitalWrite(LEDBUILTIN,LOW);
  digitalWrite(LEDPCBEXT1,LOW);
  /* configura comunicação serial com baudrate de 115200 */
  Serial.begin(115200);
  /* Criação das Tasks: Compilar 2x (comentar/descomentar o taskcreate vTAKS1) para mudar o led na passagem de parâmetros */
  xTaskCreate(vTask1,"vTASK1",configMINIMAL_STACK_SIZE,(void *)LEDBUILTIN,1,&task1Handle);
  //xTaskCreate(vTask1,"vTASK1",configMINIMAL_STACK_SIZE,(void *)LEDPCBEXT1,1,&task1Handle);
  xTaskCreate(vTask2,"vTASK2",configMINIMAL_STACK_SIZE+512,(void *)valorinicial,2,&task2Handle);
}

/* Função Loop*/
void loop()
{
  vTaskDelay(3000); //libera a CPU por 3 seg
}

/* vTask1 => inverte LED em intervalos de 200 ms */
void vTask1(void *pvParameters)
{
  int pin = (int)pvParameters;
  pinMode(pin,OUTPUT);
  while (1)
        {
          digitalWrite(pin,!digitalRead(pin));
          vTaskDelay(pdMS_TO_TICKS(200));
        }
}

/* vTask2 => imprime valor de contagem a cada 1 seg */
void vTask2(void *pvParameters)
{
  int cont = (int)pvParameters;
  while (1)
        {
          Serial.println("vTask 2: " + String(cont));
          vTaskDelay(pdMS_TO_TICKS(1000));
          cont++;
        }
}