/*###############################################################################
# Exemplo para avaliar o consumo de memória pela Task - KIT: WEMOS D1 MINI ESP32
# Esse exemplo mostra como avaliar o consumo de memória pela tarefa no FreeRTOS
# Exemplo 06: Stack / Platformio / FreeRTOS
# Por: Prof. Dr. Paulo Denis Garcez da Luz
# http://www.elf74.daeln.com.br
###############################################################################*/

/* Biblioteca do Arduino */
#include <Arduino.h>

/* Bibliotecas FreeRTOS */
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"

 /* Mapeamento de Pinos */ 
#define LEDBUILTIN 2
#define PUSHBUTTON 12
#define LEDPCBEXT1 14
#define ADCPOTPIN1 34

/* Variáveis para armazenamento do handle das tasks */
TaskHandle_t task1Handle = NULL;
TaskHandle_t task2Handle = NULL;
TaskHandle_t task3Handle = NULL;

/* Variáveis Globais*/
int valorinicial = 311;

/* Protótipos das Tasks */
void vTask1(void *pvParameters);
void vTask2(void *pvParameters);
void vTask3(void *pvParameters);

/* Função Setup */
void setup()
{
  /* configura pino 2 e 14 como saída*/
  pinMode(LEDBUILTIN,OUTPUT);
  pinMode(LEDPCBEXT1,OUTPUT);
  /* desliga os Leds da PCB */
  digitalWrite(LEDBUILTIN,LOW);
  digitalWrite(LEDPCBEXT1,LOW);
  /* configura comunicação serial com baudrate de 115200 */
  Serial.begin(115200);
  /* Criação das Tasks */
  xTaskCreatePinnedToCore(vTask1,"vTASK1",configMINIMAL_STACK_SIZE,(void *)LEDBUILTIN,1,&task1Handle,APP_CPU_NUM);
  xTaskCreatePinnedToCore(vTask2,"vTASK2",configMINIMAL_STACK_SIZE+512,(void *)valorinicial,2,&task2Handle,PRO_CPU_NUM);
  xTaskCreatePinnedToCore(vTask3,"vTASK3",configMINIMAL_STACK_SIZE+512,(void *)LEDPCBEXT1,2,&task3Handle,APP_CPU_NUM);
}

/* Função Loop*/
void loop()
{
  vTaskDelay(3000); //libera a CPU por 3 seg
}

/* vTask1 => inverte LED em intervalos de 200 ms no core principal */
void vTask1(void *pvParameters)
{
  int pin = (int)pvParameters;
  pinMode(pin,OUTPUT);
  while (1)
        {
          digitalWrite(pin,!digitalRead(pin));
          vTaskDelay(pdMS_TO_TICKS(200));
        }
}

/* vTask2 => imprime valor de contagem a cada 1 seg a task1 no core secundário */
void vTask2(void *pvParameters)
{
  UBaseType_t uxHighWaterMark;
  int cont = (int)pvParameters;
  while (1)
        {
          Serial.println("vTask2_pvParameters: " + String(cont));
          uxHighWaterMark = uxTaskGetStackHighWaterMark(NULL);
          Serial.print(pcTaskGetTaskName(NULL));
          Serial.print("_Stack: ");
          Serial.println(uxHighWaterMark);
          vTaskDelay(pdMS_TO_TICKS(1000));
          cont++;
        }
}

/* vTask3 => inverte LED em intervalos de 500 ms no core principal */
void vTask3(void *pvParameters)
{
  int pin = (int)pvParameters;
  pinMode(pin,OUTPUT);
  while (1)
        {
          digitalWrite(pin,!digitalRead(pin));
          vTaskDelay(pdMS_TO_TICKS(500));
        }
}