/*###############################################################################
# Exemplo para criar uma fila usada por duas tarefas - KIT: DOIT ESP32 DEVKIT V1
# Esse exemplo mostra colocar e retirar elementos de uma fila no FreeRTOS
# Exemplo 07: Fila / Platformio / FreeRTOS
# Por: Prof. Dr. Paulo Denis Garcez da Luz
# http://www.elf74.daeln.com.br
###############################################################################*/
#include <Arduino.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/queue.h"

 /* Mapeamento de Pinos */ 
#define LEDBUILTIN 2
#define PUSHBUTTON 12
#define LEDPCBEXT1 14
#define ADCPOTPIN1 34

/* Variáveis para armazenamento do handle das tasks */
TaskHandle_t task1Handle = NULL;
TaskHandle_t task2Handle = NULL;

/* Variáveis para armazenamento das Filasa */
QueueHandle_t xFila;

/* Protótipos das Tasks */
void vTask1(void *pvParameters);
void vTask2(void *pvParameters);

void setup() {
  /* criação da variável de retorno da criação da task */
  BaseType_t xReturned;
  /* configura pino 2 e 14 como saída*/
  pinMode(LEDBUILTIN,OUTPUT);
  pinMode(LEDPCBEXT1,OUTPUT);
  /* desliga os Leds da PCB */
  digitalWrite(LEDBUILTIN,LOW);
  digitalWrite(LEDPCBEXT1,LOW);
  /* configura comunicação serial com baudrate de 115200 */
  Serial.begin(115200);
  /* Criação da Fila com 7 posições*/
  xFila = xQueueCreate(7,sizeof(int)); 
  if (xFila == NULL)
     {
       Serial.println("Nao foi possível criar a fila");
       while(1);
     }
  /* Criação das Tasks */
  xReturned = xTaskCreate(vTask1,"Task1",configMINIMAL_STACK_SIZE+512,NULL,1,&task1Handle);
  if (xReturned == pdFAIL)
     {
      Serial.println("Não foi possível criar a Task 1");
      while(1);
     }
  xReturned = xTaskCreate(vTask2,"Task2",configMINIMAL_STACK_SIZE+512,NULL,1,&task2Handle);
  if (xReturned == pdFAIL)
     {
      Serial.println("Não foi possível criar a Task 1");
      while(1);
     }
}

/* loop-Task0 => inverte LED em intervalos de 750ms */
void loop()
{
  digitalWrite(LEDBUILTIN, !digitalRead(LEDBUILTIN));
  vTaskDelay(pdMS_TO_TICKS(7500));
}

/* vTask1 => envia o contador para a fila a cada 500ms até ele chegar em 10, então espera 5s para recomeçar */
void vTask1(void *pvParameters)
{
  int count = 0;
  while (1)
        {
          if (count<10)
             {
               xQueueSend(xFila, &count,portMAX_DELAY);
               count++;
             }
          else
             {
               count = 0;
               vTaskDelay(pdMS_TO_TICKS(5000));
             }
          vTaskDelay(pdMS_TO_TICKS(500));
        }
}

/* vTask2 => espera um dado chegar na fila e retira ele para tratamento esperando por no max. 1s */
void vTask2(void *pvParameters)
{
  int vRx = 0;
  while (1)
        {
          if (xQueueReceive(xFila,&vRx,pdMS_TO_TICKS(1000))==pdTRUE)
             {
               Serial.println("Valor Recebido da Fila: " + String(vRx));
             }
          else
             {
               Serial.println("TimeOut-Task2");
             }
        }
}