/*###############################################################################
# Exemplo para manipular uma fila a partir de uma ISR - KIT: DOIT ESP32 DEVKIT V1
# Esse exemplo mostra como colocar dados na fila através de uma ISR e FreeRTOS
# Exemplo 08: Fila / ISR / Platformio / FreeRTOS
# Por: Prof. Dr. Paulo Denis Garcez da Luz
# http://www.elf74.daeln.com.br
###############################################################################*/
#include <Arduino.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/queue.h"

 /* Mapeamento de Pinos */ 
#define LEDBUILTIN 2
#define PUSHBUTTON 12
#define LEDPCBEXT1 14
#define ADCPOTPIN1 34

/* Variáveis para armazenamento do handle das tasks */
TaskHandle_t task1Handle = NULL;

/* Variáveis para armazenamento das Filas */
QueueHandle_t xFila;

/* Protótipos das Tasks */
void vTask1(void *pvParameters);

/* Protótipos do callback da ISR do pushbutton */
void trataISR_PBT(void);

void setup() {
  /* criação da variável de retorno da criação da task */
  BaseType_t xReturned;
  /* configura pino 2 e 14 como saída*/
  pinMode(LEDBUILTIN,OUTPUT);
  pinMode(LEDPCBEXT1,OUTPUT);
  /* desliga os Leds da PCB */
  digitalWrite(LEDBUILTIN,LOW);
  digitalWrite(LEDPCBEXT1,LOW);
  /* configura pino 12 como entrada com pull-up */
  pinMode(PUSHBUTTON,INPUT_PULLUP);
  /* configura a origem da ISR e seu callback routine */
  attachInterrupt(digitalPinToInterrupt(PUSHBUTTON),trataISR_PBT,FALLING);
  /* configura comunicação serial com baudrate de 115200 */
  Serial.begin(115200);
  /* Criação da Fila com 7 posições*/
  xFila = xQueueCreate(7,sizeof(int)); 
  if (xFila == NULL)
     {
       Serial.println("Nao foi possível criar a fila");
       while(1);
     }
  /* Criação das Tasks */
  xReturned = xTaskCreate(vTask1,"Task1",configMINIMAL_STACK_SIZE+512,NULL,1,&task1Handle);
  if (xReturned == pdFAIL)
     {
      Serial.println("Não foi possível criar a Task 1");
      while(1);
     }
}

/* loop-Task0 => inverte LED em intervalos de 333ms */
void loop()
{
  digitalWrite(LEDBUILTIN, !digitalRead(LEDBUILTIN));
  vTaskDelay(pdMS_TO_TICKS(333));
}

/* vTask1 => envia quantas vezes o pushbutton foi pressionado via ISR */
void vTask1(void *pvParameters)
{
  int vRX = 0;
  while (1)
        {
          xQueueReceive(xFila,&vRX,portMAX_DELAY);
          Serial.println("BT Pressionado - via ISR xvezes: " + String(vRX));          
        }
}

/* trataISR_PBT => callback da ISR do pino do pushbutton (não faz tratamento do repique) */
void trataISR_PBT(void)
{
  static int valor;
  valor++;
  xQueueSendFromISR(xFila,&valor,NULL);
}