/*###############################################################################
# Exemplo para manipular uma semáforo binário - KIT: DOIT ESP32 DEVKIT V1
# Esse exemplo mostra como usar um semáforo binário no FreeRTOS
# Exemplo 09: Semáforo / Platformio / FreeRTOS
# Por: Prof. Dr. Paulo Denis Garcez da Luz
# http://www.elf74.daeln.com.br
###############################################################################*/
#include <Arduino.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/semphr.h"

/* Mapeamento de Pinos */ 
#define LEDBUILTIN 2
#define PUSHBUTTON 12
#define LEDPCBEXT1 14
#define ADCPOTPIN1 34

/* Variáveis para armazenamento do handle das tasks */
TaskHandle_t task1Handle = NULL;

/* Variáveis para armazenamento dos semáforos */
SemaphoreHandle_t xSemaphore;

/* Protótipos das Tasks */
void vTask1(void *pvParameters);


void setup() {
  /* criação da variável de retorno da criação da task */
  BaseType_t xReturned;
  /* configura pino 2 e 14 como saída*/
  pinMode(LEDBUILTIN,OUTPUT);
  pinMode(LEDPCBEXT1,OUTPUT);
  /* desliga os Leds da PCB */
  digitalWrite(LEDBUILTIN,LOW);
  digitalWrite(LEDPCBEXT1,LOW);
  /* configura comunicação serial com baudrate de 115200 */
  Serial.begin(115200);
  /* Criação do semáforo binário*/
  xSemaphore = xSemaphoreCreateBinary();
  if (xSemaphore == NULL)
     {
       Serial.println("Não foi possível criar o semaforo binário!");
       while(1);
     }
  /* Criação das Tasks */
  xReturned = xTaskCreate(vTask1,"Task1",configMINIMAL_STACK_SIZE+512,NULL,1,&task1Handle);
  if (xReturned == pdFAIL)
     {
      Serial.println("Não foi possível criar a Task 1!");
      while(1);
     }
}

/* loop-Task0 => inverte LED em intervalos de 2,5s e libera o semáforo binário */
void loop()
{
  digitalWrite(LEDBUILTIN,!digitalRead(LEDBUILTIN));
  Serial.println("Valor do LED: " + String(digitalRead(LEDBUILTIN)));
  vTaskDelay(pdMS_TO_TICKS(2500));
  xSemaphoreGive(xSemaphore);
}

/* vTask1 => envia o valor da leitura analógica do potenciômetro pela serial quando o semáforo binário está liberado e adquire o semáforo */
void vTask1(void *pvParameters)
{
  int adcRead = 0;
  while (1)
        {
          xSemaphoreTake(xSemaphore,portMAX_DELAY);
          adcRead = analogRead(ADCPOTPIN1);
          Serial.println("Valor do ADC: " + String(adcRead));
          Serial.println(" ");         
        }
}