/*###############################################################################
# Exemplo para usar um semáforo binário em uma ISR - KIT: WEMOS D1 MINI ESP32
# Esse exemplo mostra como usar um semáforo binário em ISR no FreeRTOS
# Exemplo 10: Semáforo / ISR / Platformio / FreeRTOS
# Por: Prof. Dr. Paulo Denis Garcez da Luz
# http://www.elf74.daeln.com.br
###############################################################################*/
#include <Arduino.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/semphr.h"

 /* Mapeamento de Pinos */ 
#define LEDBUILTIN 2
#define PUSHBUTTON 12
#define LEDPCBEXT1 14
#define ADCPOTPIN1 34

/* Variáveis para armazenamento do handle das tasks */
TaskHandle_t task1Handle = NULL;

/* Variáveis para armazenamento dos semáforos */
SemaphoreHandle_t xSemaphore;

/* Protótipos das Tasks */
void vTask1(void *pvParameters);

/* Protótipos do callback da ISR do pushbutton */
void trataISR_PBT(void);

void setup() {
  /* criação da variável de retorno da criação da task */
  BaseType_t xReturned;
  /* configura pino 2 e 14 como saída*/
  pinMode(LEDBUILTIN,OUTPUT);
  pinMode(LEDPCBEXT1,OUTPUT);
  /* desliga os Leds da PCB */
  digitalWrite(LEDBUILTIN,LOW);
  digitalWrite(LEDPCBEXT1,LOW);
  /* configura pino 12 como entrada com pull-up */
  pinMode(PUSHBUTTON,INPUT_PULLUP);
  /* configura a origem da ISR e seu callback routine */
  attachInterrupt(digitalPinToInterrupt(PUSHBUTTON),trataISR_PBT,FALLING);
  /* configura comunicação serial com baudrate de 115200 */
  Serial.begin(115200);
  /* Criação do semáforo binário*/
  xSemaphore = xSemaphoreCreateBinary();
  if (xSemaphore == NULL)
     {
       Serial.println("Não foi possível criar o semaforo binário!");
       while(1);
     }
  /* Criação das Tasks com maior prioridade para forçar troca de contexto para ela na ISR */
  xReturned = xTaskCreate(vTask1,"Task1",configMINIMAL_STACK_SIZE+512,NULL,3,&task1Handle);
  if (xReturned == pdFAIL)
     {
      Serial.println("Não foi possível criar a Task 1!");
      while(1);
     }
}

/* loop-Task0 => pisca led (100ms=ON | 900ms=OFF) */
void loop()
{
  digitalWrite(LEDBUILTIN,HIGH);
  vTaskDelay(pdMS_TO_TICKS(100));
  digitalWrite(LEDBUILTIN,LOW);
  vTaskDelay(pdMS_TO_TICKS(900));
}

/* vTask1 => aguarda pela liberação do semáforo binário e contabiliza quantas vezes o pushbutton foi pressionado enviando o valor pela serial */
void vTask1(void *pvParameters)
{
  int contador = 0;
  while (1)
        {
          xSemaphoreTake(xSemaphore,portMAX_DELAY);
          Serial.println("Push Button (pressionado): " + String(contador++));      
        }
}

/* trataISR_PBT => callback da ISR do pino do pushbutton (não faz tratamento do repique) */
void trataISR_PBT(void)
{
  BaseType_t xHighPriorityTaskWoken = pdTRUE;
  xSemaphoreGiveFromISR(xSemaphore,&xHighPriorityTaskWoken);
  if (xHighPriorityTaskWoken == pdTRUE)
     {
       portYIELD_FROM_ISR();
     }
}