/*###############################################################################
# Exemplo para manipular um mutex - KIT: WEMOS D1 MINI ESP32
# Esse exemplo mostra como usar um mutex no FreeRTOS
# Exemplo 12: Mutex / Platformio / FreeRTOS
# Por: Prof. Dr. Paulo Denis Garcez da Luz
# http://www.elf74.daeln.com.br
###############################################################################*/
#include <Arduino.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/semphr.h"

 /* Mapeamento de Pinos */ 
#define LEDBUILTIN 2
#define PUSHBUTTON 12
#define LEDPCBEXT1 14
#define ADCPOTPIN1 34

/* Variáveis para armazenamento do handle das tasks */
TaskHandle_t task1Handle = NULL;
TaskHandle_t task2Handle = NULL;

/* Variáveis para armazenamento do mutex */
SemaphoreHandle_t xMutex;

/* Protótipos das Tasks */
void vTask1(void *pvParameters);
void vTask2(void *pvParameters);

/* Protótipos das funções gerais do programa */
void vEnviaInformacao(int i);

void setup() {
  /* criação da variável de retorno da criação da task */
  BaseType_t xReturned;
  /* configura pino 2 e 14 como saída*/
  pinMode(LEDBUILTIN,OUTPUT);
  pinMode(LEDPCBEXT1,OUTPUT);
  /* desliga os Leds da PCB */
  digitalWrite(LEDBUILTIN,LOW);
  digitalWrite(LEDPCBEXT1,LOW);
  /* configura comunicação serial com baudrate de 115200 */
  Serial.begin(115200);
  /* Criação do semáforo contador de 0 a 255 */
  xMutex = xSemaphoreCreateMutex();
  if (xMutex == NULL)
     {
       Serial.println("Não foi possível criar o Mutex!");
       while(1);
     }
  /* Criação da Task1 */
  xReturned = xTaskCreate(vTask1,"Task1",configMINIMAL_STACK_SIZE+512,NULL,1,&task1Handle);
  if (xReturned == pdFAIL)
     {
      Serial.println("Não foi possível criar a Task 1!");
      while(1);
     }
  /* Criação da Task2 com maior prioridade*/
  xReturned = xTaskCreate(vTask2,"Task2",configMINIMAL_STACK_SIZE+512,NULL,4,&task2Handle);
  if (xReturned == pdFAIL)
     {
      Serial.println("Não foi possível criar a Task 2!");
      while(1);
     }
}

/* loop-Task0 => pisca led (300ms=ON | 700ms=OFF) */
void loop()
{
  digitalWrite(LEDBUILTIN,HIGH);
  vTaskDelay(pdMS_TO_TICKS(300));
  digitalWrite(LEDBUILTIN,LOW);
  vTaskDelay(pdMS_TO_TICKS(700));
}

/* vTask1 => Adquire Mutex e envia informação pela serial e congela tarefa por 2s*/
void vTask1(void *pvParameters)
{
  while (1)
        {
          xSemaphoreTake(xMutex,portMAX_DELAY);
          vEnviaInformacao(1);
          /* travar esta tarefa por 2s */
          delay(2000);
          xSemaphoreGive(xMutex);
          vTaskDelay(pdMS_TO_TICKS(100));
        }
}

/* vTask2 => Adquire Mutex e envia informação pela serial e congela tarefa por 500ms */
void vTask2(void *pvParameters)
{
  while (1)
        {
          xSemaphoreTake(xMutex,portMAX_DELAY);
          vEnviaInformacao(2);
          /* travar esta tarefa por 500ms */
          delay(500);
          xSemaphoreGive(xMutex);
          /* comentar o vTaskDelay e recompilar para verificar o que muda */
          vTaskDelay(pdMS_TO_TICKS(100));
        }
}

void vEnviaInformacao(int i)
{
  Serial.println("Recebida informação da Tarefa: " + String(i));
}