/*###############################################################################
# Exemplo para manipular softwares timers - (DevKIT: NodeMCU-32S)
# Esse exemplo mostra como usar S.T. one-shot e auto-reload FreeRTOS
# Exemplo 13: Software Timer / Platformio / FreeRTOS
# Por: Prof. Dr. Paulo Denis Garcez da Luz
# http://www.elf74.daeln.com.br
###############################################################################*/
#include <Arduino.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/timers.h"

 /* Mapeamento de Pinos */ 
#define LEDBUILTIN 2
#define PUSHBUTTON 12
#define LEDPCBEXT1 14
#define ADCPOTPIN1 34

/* Variáveis para armazenamento do handle das tasks */
TaskHandle_t task1Handle = NULL;

/* Variáveis para armazenamento dos sotwares timers */
TimerHandle_t xTimer1 = NULL;
TimerHandle_t xTimer2 = NULL;

/* Protótipos das Tasks */
void vTask1(void *pvParameters);

/* Protótipos dos callbacks dos softwares timers */
void vFTimer1(TimerHandle_t xTimer);
void vFTimer2(TimerHandle_t xTimer);

void setup() {
  /* criação da variável de retorno da criação da task */
  BaseType_t xReturned;
  /* configura pino 2 e 14 como saída*/
  pinMode(LEDBUILTIN,OUTPUT);
  pinMode(LEDPCBEXT1,OUTPUT);
  /* desliga os Leds da PCB */
  digitalWrite(LEDBUILTIN,LOW);
  digitalWrite(LEDPCBEXT1,LOW);
  /* configura pino 12 como entrada com pull-up */
  pinMode(PUSHBUTTON,INPUT_PULLUP);
  /* configura comunicação serial com baudrate de 115200 */
  Serial.begin(115200);
  /* Criação do sotware timer auto-reload de 750ms */
  xTimer1 = xTimerCreate("TIMER1",pdMS_TO_TICKS(750),pdTRUE,0,vFTimer1);
  /* Criação do sotware timer one-shot */
  xTimer2 = xTimerCreate("TIMER2",pdMS_TO_TICKS(10000),pdFALSE,0,vFTimer2);
  /* Criação da Task1 */
  xReturned = xTaskCreate(vTask1,"Task1",configMINIMAL_STACK_SIZE+512,NULL,1,&task1Handle);
  if (xReturned == pdFAIL)
     {
      Serial.println("Não foi possível criar a Task 1!");
      while(1);
     }
  /* Ativa o s.t.1 */
  xTimerStart(xTimer1,0);
}

/* loop-Task0 => somente liberar o uc para as outras tasks */
void loop()
{
  vTaskDelay(pdMS_TO_TICKS(3000));
}

/* vTask1 => faz pooling no botão com tratamento do repique, caso o botão seja pressionado ativa o timer2 e desativa o timer1 e liga o led*/
void vTask1(void *pvParameters)
{
  uint8_t repiqueTime = 0;
  while (1)
        {
          if ((digitalRead( PUSHBUTTON ) == LOW) && (xTimerIsTimerActive( xTimer2 ) == pdFALSE))
             {
               repiqueTime++;
               if (repiqueTime >= 9)
                  {
                    repiqueTime = 0;
                    digitalWrite(LEDPCBEXT1,HIGH);
                    digitalWrite(LEDBUILTIN,LOW);
                    xTimerStart(xTimer2,0);
                    xTimerStop(xTimer1,0);
                    Serial.println("Parando:(Software Timer 1) e Iniciando:(Software Timer 2)");
                  }
             }
          else repiqueTime = 0;
          vTaskDelay(pdMS_TO_TICKS(7));
        }
}

/* vFTimer1 => função de callback do s.t.1: led blinky de 1s */
void vFTimer1(TimerHandle_t xTimer)
{
   digitalWrite(LEDBUILTIN,!digitalRead(LEDBUILTIN));
}

/* vFTimer2 => função de callback do s.t.2: reativa o s.t.1 e desliga o led */
void vFTimer2(TimerHandle_t xTimer)
{
  digitalWrite(LEDPCBEXT1,LOW);
  xTimerStart(xTimer1,0);
}