/*###############################################################################
# Exemplo para manipular grupo de eventos - KIT: DOIT ESP32 DEVKIT V1
# Esse exemplo mostra como manipular event groups (Softwares Flags) no FreeRTOS
# Exemplo 14: Event Group / Platformio / FreeRTOS
# Por: Prof. Dr. Paulo Denis Garcez da Luz
# http://www.elf74.daeln.com.br
###############################################################################*/
#include <Arduino.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/timers.h"
#include "freertos/event_groups.h"

 /* Mapeamento de Pinos */ 
#define LEDBUILTIN 2
#define PUSHBUTTON 12
#define LEDPCBEXT1 14
#define ADCPOTPIN1 34

/* Mapeamento de bits para os grupos de eventos */
#define Ev_FLAG0 (1<<0)  //Bit_0 (true)
#define Ev_FLAG1 (1<<1)  //Bit_1 (true)

/* Variáveis para armazenamento do handle das tasks */
TaskHandle_t task1Handle = NULL;
TaskHandle_t task2Handle = NULL;

/* Variáveis para armazenamento dos sotwares timers */
TimerHandle_t xTimer1 = NULL;

/* Variáveis para armazenamento dos grupos de eventos (max. 24biits / variável) */
EventGroupHandle_t xEventFlags;

/* Variáveis Globais */
int tempo = 0;

/* Protótipos das Tasks */
void vTask1(void *pvParameters);
void vTask2(void *pvParameters);

/* Protótipos dos callbacks dos softwares timers */
void vFTimer1(TimerHandle_t xTimer);

void setup() {
  /* criação da variável de retorno da criação da task */
  BaseType_t xReturned;
  /* configura pino 2 e 14 como saída*/
  pinMode(LEDBUILTIN,OUTPUT);
  pinMode(LEDPCBEXT1,OUTPUT);
  /* desliga os Leds da PCB */
  digitalWrite(LEDBUILTIN,LOW);
  digitalWrite(LEDPCBEXT1,LOW);
  /* configura comunicação serial com baudrate de 115200 */
  Serial.begin(115200);
  /* Criação do grupo de eventos (24bits flags max) */
  xEventFlags = xEventGroupCreate();
  /* Criação do sotware timer auto-reload de 750ms */
  xTimer1 = xTimerCreate("TIMER1",pdMS_TO_TICKS(1000),pdTRUE,0,vFTimer1);
  /* Criação da Task1 */
  xReturned = xTaskCreate(vTask1,"Task1",configMINIMAL_STACK_SIZE+512,NULL,1,&task1Handle);
  if (xReturned == pdFAIL)
     {
      Serial.println("Não foi possível criar a Task 1!");
      while(1);
     }
  xReturned = xTaskCreate(vTask2,"Task2",configMINIMAL_STACK_SIZE+512,NULL,1,&task2Handle);
  if (xReturned == pdFAIL)
     {
      Serial.println("Não foi possível criar a Task 2!");
      while(1);
     }
  /* Ativa o s.t.1 */
  xTimerStart(xTimer1,0);
}

/* loop-Task0 => pisca led (250ms=ON | 750ms=OFF) */
void loop()
{
  digitalWrite(LEDBUILTIN,HIGH);
  vTaskDelay(pdMS_TO_TICKS(250));
  digitalWrite(LEDBUILTIN,LOW);
  vTaskDelay(pdMS_TO_TICKS(750));
}

/* vTask1 => aguarda sinalização da flag do grupo de eventos no bit 0, envia pela serial quando o evento ocorreu */
void vTask1(void *pvParameters)
{
  EventBits_t xEvBitst1;
  while (1)
        {
          xEvBitst1 = xEventGroupWaitBits(xEventFlags,Ev_FLAG0,pdTRUE,pdTRUE,portMAX_DELAY);
          Serial.println("Task 1 recebeu o flag do evento e saiu do estado de bloqueio (bit0): "+ String(xEvBitst1));
        }
}

/* vTask2 => aguarda sinalização da flag do grupo de eventos no bit 1, envia pela serial quando o evento ocorreu */
void vTask2(void *pvParameters)
{
  EventBits_t xEvBitst2;
  while (1)
        {
          xEvBitst2 = xEventGroupWaitBits(xEventFlags,Ev_FLAG1,pdTRUE,pdTRUE,portMAX_DELAY);
          Serial.println("Task 2 recebeu o flag do evento e saiu do estado de bloqueio (bit1): "+ String(xEvBitst2));
        }
}

/* vFTimer1 => função de callback do s.t.1: temporização do grupo de eventos e seus bits ... */
void vFTimer1(TimerHandle_t xTimer)
{
  if (tempo == 3) xEventGroupSetBits(xEventFlags,Ev_FLAG0);
  if (tempo == 6) xEventGroupSetBits(xEventFlags,Ev_FLAG1);
  if (tempo >= 9)
     {
       tempo = 0;
       xEventGroupSetBits(xEventFlags,Ev_FLAG0 | Ev_FLAG1);
     }
  tempo++;
}