/*###############################################################################
# Exemplo para demonstrar o uso de Task Notification - (DevKIT: NodeMCU-32S)
# Esse exemplo mostra como manipular Task Notification no FreeRTOS
# Exemplo 15: Task Notification/ ISR / Platformio / (*OBS: FreeRTOS >= 8.2)
# Por: Prof. Dr. Paulo Denis Garcez da Luz
# http://www.elf74.daeln.com.br
###############################################################################*/
#include <Arduino.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"

 /* Mapeamento de Pinos */ 
#define LEDBUILTIN 2
#define PUSHBUTTON 12
#define LEDPCBEXT1 14
#define ADCPOTPIN1 34

/* Variáveis para armazenamento do handle das tasks */
TaskHandle_t task1Handle = NULL;

/* Protótipos das Tasks */
void vTask1(void *pvParameters);

/* Protótipos do callback da ISR do pushbutton */
void trataISR_PBT(void);

void setup() {
  /* criação da variável de retorno da criação da task */
  BaseType_t xReturned;
  /* configura pino 2 e 14 como saída*/
  pinMode(LEDBUILTIN,OUTPUT);
  pinMode(LEDPCBEXT1,OUTPUT);
  /* desliga os Leds da PCB */
  digitalWrite(LEDBUILTIN,LOW);
  digitalWrite(LEDPCBEXT1,LOW);
  /* configura pino 12 como entrada com pull-up */
  pinMode(PUSHBUTTON,INPUT_PULLUP);
  /* configura a origem da ISR e seu callback routine */
  attachInterrupt(digitalPinToInterrupt(PUSHBUTTON),trataISR_PBT,FALLING);
  /* configura comunicação serial com baudrate de 115200 */
  Serial.begin(115200);
  /* Criação da Task1 */
  xReturned = xTaskCreate(vTask1,"Task1",configMINIMAL_STACK_SIZE+512,NULL,1,&task1Handle);
  if (xReturned == pdFAIL)
     {
      Serial.println("Não foi possível criar a Task 1!");
      while(1);
     }
}

/* loop-Task0 => Led Blinky (250ms=ON | 250ms=OFF) */
void loop()
{
  digitalWrite(LEDPCBEXT1,HIGH);
  vTaskDelay(pdMS_TO_TICKS(250));
  digitalWrite(LEDPCBEXT1,LOW);
  vTaskDelay(pdMS_TO_TICKS(250));
}

/* vTask1 => aguarda sinalização via task notification, inverte o led e congela por 750ms e envia pela serial o valor do contador de pressionamento do botão */
void vTask1(void *pvParameters)
{
  uint32_t qtpress = 0;
  while (1)
        {
          qtpress = ulTaskNotifyTakeIndexed(pdFALSE,0,portMAX_DELAY);
          Serial.println("Task1 - Tratando ISR botao: " + String(qtpress));
          digitalWrite(LEDBUILTIN,!digitalRead(LEDBUILTIN));
          delay(750);
        }
}

/* trataISR_PBT => callback da ISR do pino do pushbutton (não faz tratamento do repique) */
void trataISR_PBT(void)
{
  BaseType_t xHigherPriorityTaskWoken = pdFALSE;
  vTaskNotifyGiveIndexedFromISR(task1Handle,0,&xHigherPriorityTaskWoken);
}