/*###############################################################################
# Exemplo com passagem de parâmetros na Inicialização - (DevKIT: NodeMCU-32S)
# Esse exemplo mostra como passar parâmetros (struct) para as Tasks no FreeRTOS
# Exemplo 16: Tasks / Platformio / FreeRTOS
# Por: Prof. Dr. Paulo Denis Garcez da Luz
# http://www.elf74.daeln.com.br
###############################################################################*/

/* Biblioteca do Arduino */
#include <Arduino.h>

/* Bibliotecas FreeRTOS */
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"

 /* Mapeamento de Pinos */ 
#define LEDBUILTIN 2
#define PUSHBUTTON 12
#define LEDPCBEXT1 14
#define ADCPOTPIN1 34

/* Struct exemplo */
struct ablinky
{
  uint8_t pino;
  uint16_t msblinky;
};

ablinky led1 = {LEDBUILTIN,100};  //estrutura para led1: delay 1s, pino LED1(2)
ablinky led2 = {LEDPCBEXT1,200};  //estrutura para led2: delay 200ms, pino LED2(14)

/* Variáveis para armazenamento do handle das tasks */
TaskHandle_t task1Handle = NULL;
TaskHandle_t task2Handle = NULL;

/* Protótipos das Tasks */
void vTask1(void *pvParameters);
void vTask2(void *pvParameters);

void setup() {
  /* criação da variável de retorno da criação da task */
  BaseType_t xReturned;
  /* configura comunicação serial com baudrate de 115200 */
  Serial.begin(115200);
  /* Criação das Tasks */
  xReturned = xTaskCreate(vTask1,"Task1",configMINIMAL_STACK_SIZE+512,(void*)&led1,1,&task1Handle);
  if (xReturned == pdFAIL)
     {
      Serial.println("Não foi possível criar a Task 1");
      while(1);
     }
  xReturned = xTaskCreate(vTask2,"Task2",configMINIMAL_STACK_SIZE+512,(void*)&led2,1,&task2Handle);
  if (xReturned == pdFAIL)
     {
      Serial.println("Não foi possível criar a Task 1");
      while(1);
     }
}

/* loop-Task0 => Idle, devolve loop-task0 para o escalonador */
void loop()
{
  vTaskDelay(2000);
}

/* vTask1 => Blinky do LED1, recebe a struct como parêmetro */
void vTask1(void *pvParameters)
{
  ablinky *led= (ablinky *)pvParameters;
  pinMode(led->pino,OUTPUT);
  while (1)
        {
          digitalWrite(led->pino,!digitalRead(led->pino));
          vTaskDelay(pdMS_TO_TICKS(led->msblinky));
        }
}

/* vTask2 => Blinky do LED2, recebe a struct como parêmetro */
void vTask2(void *pvParameters)
{
  ablinky *led= (ablinky *)pvParameters;
  pinMode(led->pino,OUTPUT);
  while (1)
        {
          digitalWrite(led->pino,!digitalRead(led->pino));
          vTaskDelay(pdMS_TO_TICKS(led->msblinky));
        }
}